/* *********************************************************
               Function declarations
  ********************************************************* */

void initialize(void);
void timer0Init(void);
void pllInit(void);
void feed(void);
void switchLED(void);

void __attribute__ ((interrupt("IRQ"))) IRQ_Timer0 (void);

/**********************************************************
                  Header files
 **********************************************************/

#include "LPC2294.h"

/**********************************************************
                       MAIN
**********************************************************/

#define LED 1<<30
#define VIC_Timer0 4

int	main (void) {
	
	initialize();
	
	while (1) {
	}
}

/**********************************************************
                      initialize
**********************************************************/

void initialize(void){
	
	pllInit();	
	//PLL-Konfiguration (Phase Locked Loop)	
		  
	timer0Init();

	PINSEL0=0x00000000; 
	//P0.0  - P0.15 bekommen 1. Funktionalitt(GeneralPurposeInputOutput)
		
	PINSEL1=0x00000000;
	//P0.16 - P0.25 und P0.27 bis P0.31 bekommen 1. Funktionalitt(GPIO)
		
	IO0DIR = LED;
	//P0.30 wird zum Ausgang, P0.0-P0.27, P0.28, P0.29 und P0.31 werden Eingang
		
	IO0CLR = LED; 
	//P 0.30 wird Low gesetzt => LED leuchtet
	
	VICIntEnClr = 0xFFFFFFFF;
	VICIntEnable = 1<<VIC_Timer0;
	VICIntSelect = 0x00000000;
	VICVectCntl0 = 0x24;
	VICVectAddr0 = (unsigned long)IRQ_Timer0;
}

/**********************************************************
                      timer0Init
**********************************************************/

void timer0Init(void){
	
	T0PR = 0x0;
	//Prescale Register (berlaufwert) 0 => pclk = 58.9824MHz = Timer-Clock
	
	T0MCR = 0x3;	
	//TimerCounter = MatchRegister0 => Interrupt 0 + Timer Counter Reset
	
	T0MR0=58982400;
	//MatchRegister0 entspricht 0.5s
	
	T0TCR=0x2;
	//Prescale-Counter und Timer-Counter werden resettet
		
	T0TCR=0x1;
	//Prescale-Counter und Timer-Counter werden gestartet
}

/**********************************************************
                      pllInit
**********************************************************/

#define PLOCK 1<<10

void pllInit(void)  {
	 
	// 				Setting the Phased Lock Loop (PLL)
	//               ----------------------------------
	//
	// Olimex LPC-H2294 has a 14.7456 MHz crystal
	//
	// We'd like the LPC-H2294 to run at 58.9824 MHz (has to be an even multiple of crystal)
	// 
	// According to the Philips LPC2294 manual:   M = cclk / Fosc	where:	M    = PLL multiplier (bits 0-4 of PLLCFG)
	//																		cclk = 58982400 Hz
	//																		Fosc = 14745600 Hz
	//
	// Solving:	M = 58982400 / 14745600 = 4
	//
	//			Note: M - 1 must be entered into bits 0-4 of PLLCFG (assign 3 to these bits)
	//
	//
	// The Current Controlled Oscilator (CCO) must operate in the range 156 mhz to 320 mhz
	//
	// According to the Philips LPC2106 manual:	Fcco = cclk * 2 * P    where:	Fcco = CCO frequency 
	//																			cclk = 58982400 Hz
	//																			P = PLL divisor (bits 5-6 of PLLCFG)
	//
	// Solving:	Fcco = 58982400 * 2 * P
	//			P = 2  (trial value)
	//			Fcco = 58982400 * 2 * 2
	//			Fcc0 = 235929600 hz    (good choice for P since it's within the 156 mhz to 320 mhz range
	//
	// From Table 19 (page 48) of Philips LPC2106 manual    P = 2, PLLCFG bits 5-6 = 1  (assign 1 to these bits)
	//
	// Finally:      PLLCFG = 0  01  00011  =  0x23
	//
	// Final note: to load PLLCFG register, we must use the 0xAA followed 0x55 write sequence to the PLLFEED register
	//             this is done in the short function feed() below
	//
   
	// Setting Multiplier and Divider values
  	PLLCFG=0x23;
  	feed();
  
	// Enabling the PLL */	
	PLLCON=0x1;
	feed();
  
	// Wait for the PLL to lock to set frequency
	while(!(PLLSTAT & PLOCK)) ;
  
	// Connect the PLL as the clock source
	PLLCON=0x3;
	feed();
  
	// Enabling MAM and setting number of clocks used for Flash memory fetch (4 cclks in this case)
	MAMCR=0x2;
	MAMTIM=0x4;
  
	// Setting peripheral Clock (pclk) to System Clock (cclk)
	VPBDIV=0x1;
}

/**********************************************************
                      feed
**********************************************************/

void feed(void)
{
  PLLFEED=0xAA;
  PLLFEED=0x55;
}

/**********************************************************
                      switchLED
**********************************************************/

void switchLED(void){
	if(IO0PIN&LED)
		IO0CLR = LED;
	else
		IO0SET = LED;
}

/**********************************************************
                      IRQ_Timer0
**********************************************************/

void IRQ_Timer0 (void){
	switchLED();
	T0IR = 0x01;
	VICVectAddr = 0;       // Acknowledge Interrupt
}
